/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dbaccess.sql;

import java.sql.Connection;
import java.sql.SQLException;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.dbaccess.sql.command.CommandCreator;
import it.eng.spago.dbaccess.sql.command.legacy.LegacyCommandCreator;
import it.eng.spago.dbaccess.sql.command.std.DefaultCommandCreator;
import it.eng.spago.dbaccess.sql.mappers.SQLMapper;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.tracing.TracerSingleton;

/**
* Questa Classe rappresenta una Connessione al DataBase ed  responsabile della :
*
* <li>  gestione delle transazioni ( Di Default una dataConnection  auto commited )
* <li>  creazione dei comandi da eseguire
* <li>  creazione dei DataField
*
* ATTENZIONE: UNA VOLTA TERMINATO L'USO DELLA CONNESSIONE E' NECESSARIO RICHIAMARE ESPLICITAMENTE IL METODO close()
*
* @author Andrea Zoppello 
* @version 1.0
*/
public class DataConnection {
    private Connection _connection = null;
    private CommandCreator _commandCreator = null;
    private SQLMapper _sqlMapper = null;
    private boolean _closed = false;

    /**
    * Costruttore : Attenzione la connessione viene creata  invocando setAutoCommit(true).
    * @param  connection - la connessione sql alla base dell'oggetto dataConnection
    * @param  driverVersion - la versione del driver utilizzato dalla connessione
    * @param  sqlMapper - la classe per il mappinf dei dataField
    * @throws EMFInternalError - Se viene generato qualche errore durante l'inizializzazione della connessione
    */
    public DataConnection(
        Connection connection,
        String driverVersion,
        SQLMapper sqlMapper)
        throws EMFInternalError {
        try {
            _connection = connection;
            if (!isRequestDistributed())
                _connection.setAutoCommit(true);
            _commandCreator = getCommandCreator(driverVersion);
            _sqlMapper = sqlMapper;
            _closed = false;
        } // try
        catch (SQLException sqle) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "DataConnection::DataConnection:",
                sqle);
            throw new EMFInternalError(
                EMFErrorSeverity.ERROR,
                "DataConnection::DataConnection: " + sqle.getMessage());
        } // catch (SQLException sqle) try
    } // public DataConnection(Connection connection, String driverVersion,
    // SQLMapper sqlMapper) throws EMFInternalError

    /**
    * Questo metodo  utilizzato per aprire una transazione sulla connessione.
    * @throws <code>EMFInternalError</code> - Se viene generato qualche errore durante l'inizializzazzione della transazione
    */
    public void initTransaction() throws EMFInternalError {
        if (isRequestDistributed()) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "DataConnection::initTransaction: connessione distribuita !");
            return;
        } // if (isRequestDistributed())
        if (_closed)
            throw new EMFInternalError(
                EMFErrorSeverity.ERROR,
                "DataConnection::initTransaction: connessione gi chiusa");
        try {
            _connection.setAutoCommit(false);
        } // try
        catch (SQLException sqle) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "DataConnection::initTransaction:",
                sqle);
            throw new EMFInternalError(
                EMFErrorSeverity.ERROR,
                "DataConnection::initTransaction: " + sqle.getMessage());
        } // catch (SQLException sqle) try
    } // public void initTransaction() throws EMFInternalError

    /**
    * Questo metodo  utilizzato per effettuare la commit delle operazioni effettuate durante una transazione
    * sulla connessione.
    * @throws <code>EMFInternalError</code> - Se viene generato qualche errore durante il commit della transazione
    */
    public void commitTransaction() throws EMFInternalError {
        if (isRequestDistributed()) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "DataConnection::commitTransaction: connessione distribuita !");
            return;
        } // if (isRequestDistributed())
        if (_closed)
            throw new EMFInternalError(
                EMFErrorSeverity.ERROR,
                "DataConnection::commitTransaction: connessione gi chiusa");
        try {
            _connection.commit();
            _connection.setAutoCommit(true);
        } // try
        catch (SQLException sqle) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "DataConnection::commitTransaction:",
                sqle);
            throw new EMFInternalError(
                EMFErrorSeverity.ERROR,
                "DataConnection::commitTransaction: " + sqle.getMessage());
        } // catch (SQLException sqle) try
    } // public void commitTransaction() throws EMFInternalError

    /**
    * Questo metodo  utilizzato per eseguire il rollBack  delle operazioni effettuate durante una transazione
    * sulla connessione
    * @throws <code>EMFInternalError</code> - Se viene generato qualche errore durante il rollBack
    */
    public void rollBackTransaction() throws EMFInternalError {
        if (isRequestDistributed()) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "DataConnection::rollBackTransaction: connessione distribuita !");
            return;
        } // if (isRequestDistributed())
        if (_closed)
            throw new EMFInternalError(
                EMFErrorSeverity.ERROR,
                "DataConnection::rollBackTransaction: connessione gi chiusa");
        try {
            _connection.rollback();
            _connection.setAutoCommit(true);
        } // try
        catch (SQLException sqle) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "DataConnection::rollBackTransaction:",
                sqle);
            throw new EMFInternalError(
                EMFErrorSeverity.ERROR,
                "DataConnection::rollBackTransaction: " + sqle.getMessage());
        } // catch (SQLException sqle) try
    } // public void rollBackTransaction() throws EMFInternalError

    /**
    * Questo metodo  utilizzato per chiudere la connessione
    * @throws <code>EMFInternalError</code> - Se viene generato qualche errore durante la chiusura della connessione
    */
    public void close() throws EMFInternalError {
        if (_closed) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "DataConnection::close: connessione gi chiusa");
            return;
        } // if (_closed)
        _closed = true;
        try {
            if (!(_connection.isClosed())) {
                if (!isRequestDistributed()
                    && !(_connection.getAutoCommit())) {
                    _connection.rollback();
                    TracerSingleton.log(
                        Constants.NOME_MODULO,
                        TracerSingleton.WARNING,
                        "DataConnection::close: connessione aperta, rolling back !");
                    _connection.setAutoCommit(true);
                } // if (!distributeBoolean && !(_connection.getAutoCommit()))
                _connection.close();
            } // if (!(_connection.isClosed()))
        } // try
        catch (Exception sqle) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "DataConnection::close:",
                sqle);
            throw new EMFInternalError(
                EMFErrorSeverity.ERROR,
                "DataConnection::close: " + sqle.getMessage());
        } // catch (Exception sqle) try
        finally {
            _connection = null;
            _commandCreator = null;
            _sqlMapper = null;
        } // finally try
    } //public void close() throws EMFInternalError

    /**
    * Crea un SQLCommand di insert per la connessione dato il CommandString.
    * @param commandString - La stringa contenente il comando sql.
    * @return un oggetto di tipo <code>SQLCommand</code> rappresentante il comando di insert.
    */
    public SQLCommand createInsertCommand(String commandString) {
        if (_closed) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "DataConnection::createInsertCommand: connessione gi chiusa");
            return null;
        } // if (_closed)
        return _commandCreator.createInsertCommand(this, commandString);
    } //public SQLCommand createInsertCommand(String commandString)

    /**
    * Crea un SQLCommand di delete per la connessione dato il CommandString.
    * @param <code>String</code> commandString - La stringa contenente il comando sql.
    * @return un oggetto di tipo <code>SQLCommand</code> rappresentante il comando di delete .
    */
    public SQLCommand createDeleteCommand(String commandString) {
        if (_closed) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "DataConnection::createDeleteCommand: connessione gi chiusa");
            return null;
        } // if (_closed)
        return _commandCreator.createDeleteCommand(this, commandString);
    } // public SQLCommand createDeleteCommand(String commandString)

    /**
    * Crea un SQLCommand per eseguire un stored procedure  dato il CommandString.
    * @param <code>String</code> commandString - La stringa contenente il comando sql.
    * @return un oggetto di tipo <code>SQLCommand</code> rappresentante il comando per l'esecuzione di una stored procedure.
    */
    public SQLCommand createStoredProcedureCommand(String commandString) {
        if (_closed) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "DataConnection::createStoredProcedureCommand: connessione gi chiusa");
            return null;
        } // if (_closed)
        return _commandCreator.createStoredProcedureCommand(
            this,
            commandString);
    } // public SQLCommand createDeleteCommand(String commandString)

    /**
    * Crea un SQLCommand di update per la connessione dato il CommandString.
    * @param <code>String</code> commandString - La stringa contenente il comando sql.
    * @return un oggetto di tipo <code>SQLCommand</code> rappresentante il comando di update.
    */
    public SQLCommand createUpdateCommand(String commandString) {
        if (_closed) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "DataConnection::createUpdateCommand: connessione gi chiusa");
            return null;
        } // if (_closed)
        return _commandCreator.createUpdateCommand(this, commandString);
    } // public SQLCommand createUpdateCommand(String commandString)

    /**
    * Crea un SQLCommand di select per la connessione dato il CommandString.
    * @param <code>String</code> commandString - La stringa contenente il comando sql.
    * @return un oggetto di tipo <code>SQLCommand</code> rappresentante il comando di select.
    */
    public SQLCommand createSelectCommand(String commandString) {
        return createSelectCommand(commandString, true);
    } // public SQLCommand createSelectCommand(String commandString)

    public SQLCommand createSelectCommand(
        String commandString,
        boolean scroll) {
        if (_closed) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "DataConnection::createSelectCommand: connessione gi chiusa");
            return null;
        } // if (_closed)
        return _commandCreator.createSelectCommand(this, commandString, scroll);
    } // public SQLCommand createSelectCommand(String commandString, boolean scroll)

    /**
    * Crea un DataField associandolo all' SqlMapper della connessione.
    * @param <code>String</code>  name - La stringa contenente il comando sql.
    * @param int type - il java.sql.Type del DataField.
    * @param <code>Object value</code> - valore del DataField.
    * @return un oggetto di tipo <code>DataField</code>
    */
    public DataField createDataField(String name, int type, Object value) {
        if (_closed) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "DataConnection::createDataField: connessione gi chiusa");
            return null;
        } // if (_closed)
        if (value instanceof String)
            return new DataField(name, type, (String) value, _sqlMapper);
        else
            return new DataField(name, type, value, _sqlMapper);
    } // public DataField createDataField(String name, int type, Object value)
    /**
    * Crea un DataField associandolo all' SqlMapper della connessione.
    * @param <code>String</code>  name - La stringa contenente il comando sql.
    * @param int type - il java.sql.Type del DataField.
    * @param <code>Object value</code> - valore del DataField.
    * @param <code>String typeName</code> - fully-qualified name di un SQL structured type.
    * @return un oggetto di tipo <code>DataField</code>
    */
    //Modifica Monica del 27/01/2004 - inizio
    public DataField createDataField(
        String name,
        int type,
        Object value,
        String typeName) {
        if (_closed) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "DataConnection::createDataField: connessione gi chiusa");
            return null;
        } // if (_closed)
        if (value instanceof String)
            return new DataField(
                name,
                type,
                (String) value,
                _sqlMapper,
                typeName);
        else
            return new DataField(name, type, value, _sqlMapper, typeName);
    } //public DataField createDataField(String name, int type, String value, String typeName)
    /**
    * Crea un DataField associandolo all'SqlMapper della connessione.
    * @param <code>String</code>  name - La stringa contenente il comando sql.
    * @param int type - il java.sql.Type del DataField.
    * @param <code>String value</code> - valore del DataField.
    * @param <code>String typeName</code> - fully-qualified name di un SQL structured type.
    * @return un oggetto di tipo <code>DataField</code>
    */
    public DataField createDataField(
        String name,
        int type,
        String value,
        String typeName) {
        if (_closed) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "DataConnection::createDataField: connessione gi chiusa");
            return null;
        } // if (_closed)
        return new DataField(name, type, value, _sqlMapper, typeName);
    } //public DataField createDataField(String name, int type, String value, String typeName) 
    //Modifica Monica del 27/01/2004 - fine
    /**
    * Crea un DataField associandolo all'SqlMapper della connessione.
    * @param <code>String</code>  name - La stringa contenente il comando sql.
    * @param int type - il java.sql.Type del DataField.
    * @param <code>String value</code> - valore del DataField.
    * @return un oggetto di tipo <code>DataField</code>
    */
    public DataField createDataField(String name, int type, String value) {
        if (_closed) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "DataConnection::createDataField: connessione gi chiusa");
            return null;
        } // if (_closed)
        return new DataField(name, type, value, _sqlMapper);
    } // public DataField createDataField(String name, int type, Object value)

    /**
    * Questo metodo permette di recuperare l'oggetto che rappresenta la tipologia di Mapper
    * associata a quella connessione
    * @return l'oggetto di tipo <code>SQLMapper</code> associato alla connessione
    */
    public SQLMapper getSQLMapper() {
        if (_closed) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "DataConnection::getSQLMapper: connessione gi chiusa");
            return null;
        } // if (_closed)
        return _sqlMapper;
    } // public SQLMapper getSQLMapper()

    /**
    * Metodo per ottenere il livello di isolamento associato al DataConnection.
    * @return int il valore di una delle seguenti costanti :
    * <li> TRANSACTION_READ_UNCOMMITTED
    * <li> TRANSACTION_READ_COMMITTED
    * <li> TRANSACTION_REPEATABLE_READ
    * <li> TRANSACTION_SERIALIZABLE
    */
    public int getTransactionIsolation() throws EMFInternalError {
        if (_closed)
            throw new EMFInternalError(
                EMFErrorSeverity.ERROR,
                "DataConnection::getTransactionIsolation: connessione gi chiusa");
        try {
            return _connection.getTransactionIsolation();
        } // try
        catch (SQLException sqle) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "DataConnection::getTransactionIsolation:",
                sqle);
            throw new EMFInternalError(
                EMFErrorSeverity.ERROR,
                "DataConnection::getTransactionIsolation: "
                    + sqle.getMessage());
        } // catch (SQLException sqle) try
    } // public int getTransactionIsolation() throws EMFInternalError

    /**
    * Metodo per impostare il livello di isolamento associato al DataConnection.
    * @param int transactionIsolationLevel il valore di una delle seguenti costanti :
    * <li> TRANSACTION_READ_UNCOMMITTED
    * <li> TRANSACTION_READ_COMMITTED
    * <li> TRANSACTION_REPEATABLE_READ
    * <li> TRANSACTION_SERIALIZABLE
    */
    public void setTransactionIsolation(int transactionIsolationLevel)
        throws EMFInternalError {
        if (_closed)
            throw new EMFInternalError(
                EMFErrorSeverity.ERROR,
                "DataConnection::setTransactionIsolation: connessione gi chiusa");
        try {
            _connection.setTransactionIsolation(transactionIsolationLevel);
        } // try
        catch (SQLException sqle) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "DataConnection::setTransactionIsolation:",
                sqle);
            throw new EMFInternalError(
                EMFErrorSeverity.ERROR,
                "DataConnection::setTransactionIsolation: "
                    + sqle.getMessage());
        } // catch (SQLException sqle)
    } // public void setTransactionIsolation(int transactionIsolationLevel) throws EMFInternalError

    /**
    * Metodo per ottenere la connessione .
    * @return <code>Connection</code> la connessione attiva.
    */
    public Connection getInternalConnection() {
        if (_closed) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "DataConnection::getInternalConnection: connessione gi chiusa");
            return null;
        } // if (_closed)
        return _connection;
    } // public Connection getInternalConnection()

    private CommandCreator getCommandCreator(String driverVersion) {
        float driverVersionFloat = Float.parseFloat(driverVersion);
        if (driverVersionFloat >= 2.0)
            return new DefaultCommandCreator();
        else
            return new LegacyCommandCreator();
    } // private CommandCreator getCommandCreator(String driverVersion)

    private boolean isRequestDistributed() {
        boolean distributedBoolean = false;
        RequestContainer requestContainer =
            RequestContainer.getRequestContainer();
        if (requestContainer != null)
            distributedBoolean = requestContainer.isRequestDistributed();
        return distributedBoolean;
    } // private boolean isRequestDistributed()

    /*
    protected void finalize() {
        if (!_closed) {
            try {
                close();
            } // try
            catch (Exception ex) {
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.CRITICAL,
                    "DataConnection::finalize:",
                    ex);
            } // catch (Exception ex)
        } // if (!_closed)
    } // protected void finalize()
    */
} // public class DataConnection
