/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.dispatching.service;

import java.io.Serializable;

import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.error.EMFErrorHandler;

/**
 * Questa classe incapsula il contesto di una richiesta di servizio.
 * Le classi che estendono <code>DefaultRequestContext</code> ereditano gli attributi che rappresentano
 * il servizio (<code>RequestContainer</code>,<code>ResponseContainer</code>) e gli attributi che gli permettono di
 * gestirlo (<code>EMFErrorHandler</code>).
 */
public class DefaultRequestContext implements RequestContextIFace, Serializable {
    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    private RequestContainer _requestContainer = null;
    private ResponseContainer _responseContainer = null;

    /**
     * Costruisce un <code>DefaultRequestContext</code> avente attributi non definiti.
     * <p>
     */
    public DefaultRequestContext() {
        super();
        _requestContainer = null;
        _responseContainer = null;
    } // public DefaultRequestContext()

    /**
     * Costruisce un <code>DefaultRequestContext</code> avente attributi definiti dai parametri
     * passati.
     * @param requestContainer l'istanza da cui impostare lo stato.
     * @param responseContainer l'istanza da cui impostare lo stato.
     */
    public DefaultRequestContext(RequestContainer requestContainer, ResponseContainer responseContainer) {
        super();
        _requestContainer = requestContainer;
        _responseContainer = responseContainer;
    } // public DefaultRequestContext(RequestContainer requestContainer,
    // ResponseContainer responseContainer)

    /**
     * Imposta lo stato della classe partendo da un 'istanza di <code>RequestContextIFace</code>
     * @param requestContext l'istanza da cui impostare lo stato.
     * @see RequestContainer
     * @see ResponseContainer
     */
    public void setRequestContext(RequestContextIFace requestContext) {
        if (requestContext == null) {
            _requestContainer = null;
            _responseContainer = null;
        } // if (requestContext == null)
        else {
            _requestContainer = requestContext.getRequestContainer();
            _responseContainer = requestContext.getResponseContainer();
        } // if (requestContext == null) else
    } // public void setRequestContext(RequestContextIFace requestContext)

    /**
     * Ritorna il <code>RequestContainer</code> attributo di classe.
     * <p>
     * @see RequestContainer
     * @return il <code>RequestContainer</code> attributo di classe.
     */
    public RequestContainer getRequestContainer() {
        return _requestContainer;
    } // public RequestContainer getRequestContainer()

    /**
     * Ritorna il <code>ResponseContainer</code> attributo di classe.
     * @return il <code>ResponseContainer</code> attributo di classe.
     * @see ResponseContainer
     */
    public ResponseContainer getResponseContainer() {
        return _responseContainer;
    } // public ResponseContainer getResponseContainer()

    /**
     * Ritorna il <code>EMFErrorHandler</code> gestore degli errori per una richiesta di servizio.
     * <p>
     * @return il <code>EMFErrorHandler</code> gestore degli errori.
     */
    public EMFErrorHandler getErrorHandler() {
        if (_responseContainer == null)
            return null;
        return _responseContainer.getErrorHandler();
    } // public EngErrorHadler getErrorHandler()

    /**
     * Ritorna il <code>SourceBean</code> contenente tutti i parametri della richiesta di servizio.
     * @return il <code>SourceBean</code> contenente i parametri della richiesta.
     */
    public SourceBean getServiceRequest() {
        if (_requestContainer == null)
            return null;
        return _requestContainer.getServiceRequest();
    } // public SourceBean getServiceRequest()

    /**
     * Ritorna il <code>SourceBean</code> contenente gli oggetti istanziati nella business logic in risposta
     * ad una richiesta di servizio.
     * @return il <code>SourceBean</code> contenente gli oggetti di risposta.
     */
    public SourceBean getServiceResponse() {
        if (_responseContainer == null)
            return null;
        return _responseContainer.getServiceResponse();
    } // public SourceBean getServiceResponse()
} // public class DefaultRequestContext implements RequestContextIFace, Serializable
