/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.event.manager;

import java.util.ArrayList;
import java.util.List;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dbaccess.DataConnectionManager;
import it.eng.spago.dbaccess.Utils;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.QueryExecutor;

/**
 * @author Daniela Butano
 *
 * This class implements all methods (defined in EventManagerIFace)
 * to manage events
 */
public class EventManager implements EventManagerIFace {
    protected EventsFacadeIFace _eventsFacade = null;

	/**
	 * Constructor for EventHandler.
	 */
    public EventManager(EventsFacadeIFace eventsFacade) {
        _eventsFacade = eventsFacade;
    }

     /**
    * This method provide to return a SourceBean representing all events
    * This method throws an exception if something goes wrong
    * @return a SourceBean of all event
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public SourceBean getAllEvents() throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "EventManager::getAllEvents:invocato");
        try {
            SourceBean resultBean = QueryExecutor.executeProfilingQuery(QueryExecutor.READ, "GET_ALL_EVENTS", null);
            return resultBean;
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "EventManager::getAllEvents:", ex);
            throw ex;
        }
    }

     /**
    * This method provide to return a SourceBean representing all events with the type identified by typeEvent
    * If typeEvent is equals to "", the method return all events
    * This method throws an exception if something goes wrong
    * @return a SourceBean of all events
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public SourceBean getEventsForType(String typeEvent) throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "EventManager::getEventsForType:invocato");
        try {
            ArrayList inputParameters = new ArrayList();
            if (typeEvent.equals(""))
                typeEvent = "%";
            inputParameters.add(typeEvent);
            SourceBean resultBean = QueryExecutor.executeProfilingQuery(QueryExecutor.READ,
                "GET_EVENTS_FOR_TYPE", inputParameters);
            return resultBean;
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "EventManager::getEventsForType:", ex);
            throw ex;
        }
    }

    /**
    * This method provide to return a SourceBean of plugins associable with the event identified by eventName
    * This method throws an exception if something goes wrong
    * @return a SourceBean of plugins
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public SourceBean getPluginsAssociableWithEvent(String eventName) throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "EventManager::getPluginsAssociableWithEvent:invocato");
        try {
            ArrayList inputParameters = new ArrayList();
            inputParameters.add(eventName);
            SourceBean resultBean = QueryExecutor.executeProfilingQuery(QueryExecutor.READ,
                "GET_PLUGINS_ASSOCIABLE_WITH_EVENT", inputParameters);
            return resultBean;
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "EventManager::getPluginsAssociableWithEvent:", ex);
            throw ex;
        }
    }

    /**
    * This method provide to return a SourceBean representing plugin with type identified by pluginType
    * and that is associable with an event identified by eventName
    * This method throws an exception if something goes wrong
    * @return a SourceBean representing the plugin
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public SourceBean getPluginAssociableWithEvent(String eventName, String pluginType) throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "EventManager::getPluginsAssociableWithEvent:invocato");
        try {
            ArrayList inputParameters = new ArrayList();
            inputParameters.add(eventName);
            inputParameters.add(pluginType);
            SourceBean resultBean = QueryExecutor.executeProfilingQuery(QueryExecutor.READ,
                "GET_PLUGINS_ASSOCIABLE_WITH_EVENT_PLUGIN_TYPE", inputParameters);
            return resultBean;
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "EventManager::getPluginsAssociableWithEvent:", ex);
            throw ex;
        } // catch
    }

    /**
    * This method provide to associate a Plugin to an Event. It's not possibile to add two plugins
    * of the same type to an event (throws exception)
    * This method throws an exception if something goes wrong
    *
    * @param pluginName
    * @param eventName
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public void associatePluginToEvent(String pluginName, String eventName) throws Exception {
        DataConnectionManager dataConnectionManager = null;
        DataConnection dataConnection = null;
        ConfigSingleton config = ConfigSingleton.getInstance();
        try {
            dataConnectionManager = DataConnectionManager.getInstance();
            String pool = (String)config.getAttribute("events_pool");
            dataConnection = dataConnectionManager.getConnection(pool);
            dataConnection.initTransaction();
            //retrieve the plugin's type
            String query = "GET_PLUGIN_TYPE_FOR_PLUGIN";
            ArrayList inputParameters = new ArrayList();
            inputParameters.add(pluginName);
            SourceBean resultBean = QueryExecutor.executeProfilingQuery(dataConnection, QueryExecutor.READ,
                query, inputParameters);
            String typePlugin = (String)((SourceBean)resultBean.getAttribute("ROW")).getAttribute(EventsConstants.PLUGIN_TYPE);
            //before to add the plugin, check if there is another plugin of the same type
            inputParameters = new ArrayList();
            inputParameters.add(eventName);
            inputParameters.add(typePlugin);
            query = "GET_PLUGINS_ASSOCIABLE_WITH_EVENT_PLUGIN_TYPE";
            resultBean = QueryExecutor.executeProfilingQuery(dataConnection, QueryExecutor.READ, query, inputParameters);
            if (resultBean.getAttributeAsList("ROW").size() == 0) {
                //if there is'nt, add plugin
                inputParameters = new ArrayList();
                inputParameters.add(pluginName);
                inputParameters.add(eventName);
                query = "ASSOCIATE_PLUGIN_TO_EVENT";
                resultBean = QueryExecutor.executeProfilingQuery(dataConnection, QueryExecutor.CREATE, query, inputParameters);
            } // if ( resultBean.getAttributeAsVector("ROW").size() == 0)
            else
                throw new EMFUserError(EMFErrorSeverity.ERROR, 10103);
            dataConnection.commitTransaction();
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "EventManager::associatePluginToEvent:", ex);
            dataConnection.rollBackTransaction();
            throw ex;
        } // catch (Exception ex) try
        finally {
            Utils.releaseResources(dataConnection, null, null);
        } // finally try                      	
    }

    /**
    * This method provide to remove association between a Plugin and an Event.
    * (and all triggers associated to plugin and event)
    * This method throws an exception if something goes wrong
    *
    * @param pluginName
    * @param eventName
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public void removePluginFromEvent(String pluginName, String eventName) throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "EventManager::removePluginFromEvent:invocato");
        DataConnectionManager dataConnectionManager = null;
        DataConnection dataConnection = null;
        ConfigSingleton config = ConfigSingleton.getInstance();
        try {
            dataConnectionManager = DataConnectionManager.getInstance();
            String pool = (String)config.getAttribute("events_pool");
            dataConnection = dataConnectionManager.getConnection(pool);
            dataConnection.initTransaction();
            //remove association between plugin and event
            String query = "REMOVE_PLUGIN_FROM_EVENT";
            ArrayList inputParameters = new ArrayList();
            inputParameters.add(pluginName);
            inputParameters.add(eventName);
            QueryExecutor.executeProfilingQuery(dataConnection, QueryExecutor.DELETE, query, inputParameters);
            //remove all triggers associated to plugin, event  			
            query = "REMOVE_TRIGGER_WITH_PLUGIN_EVENT";
            QueryExecutor.executeProfilingQuery(dataConnection, QueryExecutor.DELETE, query, inputParameters);
            dataConnection.commitTransaction();
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "EventManager::removePluginFromEvent:", ex);
            dataConnection.rollBackTransaction();
            throw ex;
        } // catch (Exception ex) try
        finally {
            Utils.releaseResources(dataConnection, null, null);
        } // finally try                      	
    }

    /**
    * This method provide to return a SourceBean representing  all event's typologies
    * This method throws an exception if something goes wrong
    * @return a SourceBean representing  all event's typologies
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public SourceBean getEventsType() throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "EventManager::getEventsType:invocato");
        try {
            SourceBean resultBean = QueryExecutor.executeProfilingQuery(QueryExecutor.READ, "GET_EVENT_TYPE", null);
            return resultBean;
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "EventManager::getEventsType:", ex);
            throw ex;
        }
    }

    /**
    * This method return a SourceBean representing events associated with this role
    * This method throws an exception if something goes wrong
    *
    * @param String roleName
    * @return A SourceBean of events for roles
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public SourceBean getEventsForRole(String roleName) throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "EventManager::getEventsForRole:invocato");
        try {
            ArrayList inputParameters = new ArrayList();
            inputParameters.add(roleName);
            SourceBean resultBean = QueryExecutor.executeProfilingQuery(QueryExecutor.READ,
                "GET_EVENTS_FOR_ROLE", inputParameters);
            return resultBean;
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "EventManager::getEventsForRole:", ex);
            throw ex;
        }
    }

    /**
    * This method return the array list rapresenting the events associated with this user
    * This method throws an exception if something goes wrong
    *
    * @param userId
    * @retrurn The list of events for user
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public SourceBean getEventsForUser(String userId) throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "EventManager::getEventsForUser:invocato");
        SourceBean eventsForUser = new SourceBean("ROWS");
        try {
            //recupero ruoli associati all'utente
            ArrayList rolesForUser = _eventsFacade.getAllRoles(userId);
            List eventsForRole = new ArrayList();
            SourceBean eventBean = null;
            String eventKey = null;
            //per ogni ruolo recupero gli eventi
            for (int indexRole = 0; indexRole < rolesForUser.size(); indexRole++) {
                eventsForRole = getEventsForRole((String)rolesForUser.get(indexRole)).getAttributeAsList("ROW");
                //....e li aggiungo a eventsForUser (dopo aver controllato
                // che non siano gi presenti)
                for (int i = 0; i < eventsForRole.size(); i++) {
                    eventBean = (SourceBean)eventsForRole.get(i);
                    eventKey = (String)(eventBean.getAttribute(EventsConstants.EVENT_KEY));
                    if (eventsForUser.getFilteredSourceBeanAttribute("ROW", EventsConstants.EVENT_KEY, eventKey) == null)
                        eventsForUser.setAttribute(eventBean);
                }
            }
            return eventsForUser;
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "EventManager::getEventsForUser:", ex);
            throw ex;
        }
    }

    /**
    * This method provide to return a SourceBean representing roles associated with the event specified by event name
    * This method throws an exception if something goes wrong
    *
    * @param String eventName
    * @return SourceBean representing roles associated with this event
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public SourceBean getRolesForEvent(String eventName) throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "EventManager::getRolesForEvent:invocato");
        try {
            ArrayList inputParameters = new ArrayList();
            inputParameters.add(eventName);
            SourceBean resultBean = QueryExecutor.executeProfilingQuery(QueryExecutor.READ,
                "GET_ROLES_FOR_EVENT", inputParameters);
            return resultBean;
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "EventManager::getRolesForEvent:", ex);
            throw ex;
        } // catch
    }

    /**
    * This method provide to remove association between Role and Event
    * This method throws an exception if something goes wrong
    *
    * @param String roleName
    * @param String eventName
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public void removeRoleFromEvent(String roleName, String eventName) throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "EventManager::removeRoleFromEvent: invocato");
        if (roleName != null && eventName != null) {
            DataConnectionManager dataConnectionManager = null;
            DataConnection dataConnection = null;
            ConfigSingleton config = ConfigSingleton.getInstance();
            try {
                dataConnectionManager = DataConnectionManager.getInstance();
                String pool = (String)config.getAttribute("events_pool");
                dataConnection = dataConnectionManager.getConnection(pool);
                dataConnection.initTransaction();
                ArrayList inputParameters = new ArrayList();
                inputParameters.add(roleName);
                inputParameters.add(eventName);
                QueryExecutor.executeProfilingQuery(dataConnection, QueryExecutor.DELETE,
                    "REMOVE_ROLE_FROM_EVENT", inputParameters);
                QueryExecutor.executeProfilingQuery(dataConnection, QueryExecutor.DELETE,
                    "REMOVE_TRIGGER_WITH_ROLE_EVENT", inputParameters);
                dataConnection.commitTransaction();
            } // try
            catch (Exception ex) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "EventManager::removeRoleFromEvent", ex);
                dataConnection.rollBackTransaction();
                throw ex;
            } // catch
            finally {
                Utils.releaseResources(dataConnection, null, null);
            } // finally
        } // if (roleName != null && eventName != null )
        else
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "EventManager::removeRoleFromEvent: roleName o eventName null");
    }

    /**
    * This method provide to associate a Role to an Event.
    * This method throws an exception if something goes wrong
    *
    * @param String roleNme
    * @param String eventName
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public void associateRoleToEvent(String roleName, String eventName) throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "EventManager::associateRoleToEvent: invocato");
        if (roleName != null && eventName != null) {
            try {
                ArrayList inputParameters = new ArrayList();
                inputParameters.add(roleName);
                inputParameters.add(eventName);
                QueryExecutor.executeProfilingQuery(QueryExecutor.CREATE,
                    "ASSOCIATE_ROLE_TO_EVENT", inputParameters);
            } // try
            catch (Exception ex) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "EventManager::associateRoleToEvent", ex);
                throw ex;
            }
        } // if (roleName != null && eventName != null )
        else
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "EventManager::associateRoleToEvent: roleName o eventName null");
    }

    /**
    * This method provide to create a New event type
    * This method throws an exception if something goes wrong
    *
    * @param String key, String description of event type to insert
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public void createNewEventType(String key, String description) throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "EventManager::createNewEventType: invocato");
        if (key != null && description != null) {
            try {
                ArrayList inputParameters = new ArrayList();
                inputParameters.add(key);
                inputParameters.add(description);
                QueryExecutor.executeProfilingQuery(QueryExecutor.CREATE,
                    "STORE_EVENT_TYPE", inputParameters);
            } // try
            catch (Exception ex) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "EventManager::createNewEventType", ex);
                throw ex;
            }
        } // if (key != null && description != null )
        else
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "EventManager::createNewEventType: typeName null");
    }

    /**
    * This method provide to remove the event type
    * This method throws an exception if something goes wrong or if there are
    * some events associated
    *
    * @param String key of event type to remove
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public void removeEventType(String key) throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "EventManager::removeEventType: invocato");
        if (key != null) {
            DataConnectionManager dataConnectionManager = null;
            DataConnection dataConnection = null;
            SourceBean resultBean = null;
            ConfigSingleton config = ConfigSingleton.getInstance();
            ArrayList inputParameters = new ArrayList();
            String query = null;
            try {
                dataConnectionManager = DataConnectionManager.getInstance();
                String pool = (String)config.getAttribute("events_pool");
                dataConnection = dataConnectionManager.getConnection(pool);
                dataConnection.initTransaction();
                //verifico che non ci siano eventi associati,
                query = "GET_EVENTS_FOR_TYPE_LOCK";
                inputParameters.add(key);
                resultBean = QueryExecutor.executeProfilingQuery(dataConnection, QueryExecutor.READ, query, inputParameters);
                //se non ci sono eventi associati viene rimosso il tipo evento
                if (resultBean.getAttributeAsList("ROW").size() == 0) {
                    query = "REMOVE_EVENT_TYPE";
                    resultBean = QueryExecutor.executeProfilingQuery(dataConnection, QueryExecutor.DELETE,
                        query, inputParameters);
                } // if (resultBean.getAttributeAsVector("ROW").size() == 0)
                else
                    throw(new EMFUserError(EMFErrorSeverity.ERROR, 10101));
                dataConnection.commitTransaction();
            } // try
            catch (Exception ex) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "EventManager::removeEventType:", ex);
                dataConnection.rollBackTransaction();
                throw ex;
            } // catch (Exception ex) try
            finally {
                Utils.releaseResources(dataConnection, null, null);
            } // finally try
        } // if ( key != null)
        else
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "EventManager::removeEventType: key vale null");
    }

    /**
    * This method provide to create a New event
    * This method throws an exception if something goes wrong
    *
    * @param String key, String description, String type of event to insert
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public void createNewEvent(String key, String description, String type) throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "EventManager::createNewEvent: invocato");
        if (key != null && description != null && type != null) {
            try {
                ArrayList inputParameters = new ArrayList();
                inputParameters.add(key);
                inputParameters.add(description);
                inputParameters.add(type);
                QueryExecutor.executeProfilingQuery(QueryExecutor.CREATE,
                    "STORE_EVENT", inputParameters);
            } // try
            catch (Exception ex) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "EventManager::createNewEvent:", ex);
                throw ex;
            } // catch (Exception ex) try
        } // if( key != null && description != null && type != null)
        else
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "EventManager::createNewEvent: key o description o type sono null");
    }

    /**
    * This method provide to remove the event identified by key, all associations with plugins, roles
    * and all triggers associated to it
    * This method throws an exception if something goes wrong
    *
    * @param key of event to remove
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public void removeEvent(String key) throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "EventManager::removeEvent: invocato");
        if (key != null) {
            DataConnectionManager dataConnectionManager = null;
            DataConnection dataConnection = null;
            ConfigSingleton config = ConfigSingleton.getInstance();
            ArrayList inputParameters = new ArrayList();
            String query = null;
            try {
                dataConnectionManager = DataConnectionManager.getInstance();
                String pool = (String)config.getAttribute("events_pool");
                dataConnection = dataConnectionManager.getConnection(pool);
                dataConnection.initTransaction();
                //rimuovo tutti i trigger associati all'evento
                query = "REMOVE_TRIGGER_WITH_EVENT";
                inputParameters.add(key);
                QueryExecutor.executeProfilingQuery(dataConnection, QueryExecutor.DELETE, query, inputParameters);
                //rimuovo tutte le associazioni con i ruoli
                query = "REMOVE_ROLE_FROM_EVENT_WITH_EVENT";
                QueryExecutor.executeProfilingQuery(dataConnection, QueryExecutor.DELETE, query, inputParameters);
                //rimuovo tutte le associazioni con i plugin	  			
                query = "REMOVE_PLUGIN_FROM_EVENT_WITH_EVENT";
                QueryExecutor.executeProfilingQuery(dataConnection, QueryExecutor.DELETE, query, inputParameters);
                //rimuovo l'evento
                query = "REMOVE_EVENT";
                QueryExecutor.executeProfilingQuery(dataConnection, QueryExecutor.DELETE, query, inputParameters);
                dataConnection.commitTransaction();
            } // try
            catch (Exception ex) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "EventManager::removeEvent:", ex);
                dataConnection.rollBackTransaction();
                throw ex;
            } // catch (Exception ex) try
            finally {
                Utils.releaseResources(dataConnection, null, null);
            } // finally try
        } // if (key != null)
        else
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "EventManager::removeEvent: key vale null");
    }
}
