/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.tracing;

import it.eng.spago.base.SourceBean;
import it.eng.spago.base.XMLObject;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.init.InitializerIFace;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import com.jamonapi.Monitor;
import com.jamonapi.MonitorFactory;
/**
* DATE            CONTRIBUTOR/DEVELOPER    NOTE
* 13-12-2004		  Butano           SourceBean ora restituisce ArrayList
* 	
* DATE            CONTRIBUTOR/DEVELOPER    NOTE
* 22-04-2005		  Bernabei          Gestito il Logger di default									    
**/
/**
 * Questo oggetto ha la responsabilit di tracciare un messaggio su file di log,
 * filtrando eventualmente quelli con severity inferiore a quella massima
 * configurata.
 */
 

public final class TracerSingleton {
    public static final int INFORMATION = 0;
    public static final int WARNING = 1;
    public static final int MINOR = 2;
    public static final int MAJOR = 3;
    public static final int CRITICAL = 4;
    public static final int DEBUG = 5;
    private static IFaceLogger _defaultLogger = null;
    private static HashMap _loggerRegistry = null;
    private static boolean _isEnabled = false;


    private TracerSingleton() {
        super();
        _defaultLogger = null;
        _loggerRegistry = null;
        _isEnabled = false;
    } // private TracerSingleton()

    /**
     * Abilita il tracciamento.
     */
    public static void enable() {
        synchronized (TracerSingleton.class) {
            //accede al file tracing.xml e per ogni busta <LOGGER> 
            //istanzia la classe corrispondente
            ConfigSingleton configure = ConfigSingleton.getInstance();
            List loggers =
                 configure.getAttributeAsList("TRACING.LOGGER");
            // number of configured logger 
            int numberOfLogger=-1;
            if (loggers!=null){
            	numberOfLogger=loggers.size();
            }
            Iterator iter = loggers.iterator();
            if (_loggerRegistry == null) {
                _loggerRegistry = new HashMap();
                while (iter.hasNext()) {
                    SourceBean loggerBean = (SourceBean) iter.next();
                    String loggerName =
                        (String) loggerBean.getAttribute("NAME");
                    String loggerDefault =
                        (String) loggerBean.getAttribute("isDefault");                    
                    if (loggerName == null)
                        continue;
                    IFaceLogger logger =
                        (IFaceLogger) _loggerRegistry.get(
                            loggerName.toUpperCase());
                    if (logger == null) {
                        String loggerClass =
                            (String) loggerBean.getAttribute("CLASS");
                        logger = FactoryLogger.getLogger(loggerClass);
                        if (logger != null) {
                            //se il logger  istanza di InitializerIFace, allora gli passa la sua configurazione
                            logger.setName(loggerName);
                            // imposto il default
                            if ( ( numberOfLogger==1  ) || // se esiste un solo logger lo imposto come il default
                                 (loggerDefault!=null && loggerDefault.equalsIgnoreCase("true"))){
                            	_defaultLogger=logger;
                            }
                            if (logger instanceof InitializerIFace) {
                                SourceBean config =
                                    (SourceBean) loggerBean.getAttribute(
                                        "CONFIG");
                                ((InitializerIFace) logger).init(config);
                            } // if (logger instanceof InitializerIFace)
                            _loggerRegistry.put(
                                loggerName.toUpperCase(),
                                logger);
                        } // if (logger != null)
                    } // if (logger == null)
                } //while(iter.hasNext())
            } //if (_loggerRegistry == null)
            Iterator names = (_loggerRegistry.keySet()).iterator();
            while (names.hasNext()) {
                IFaceLogger logger =
                    (IFaceLogger) _loggerRegistry.get((String) names.next());
                try {
                    logger.enable();
                } // try
                catch (Exception ex) {
                    ex.printStackTrace();
                } // catch (Exception ex)
            } //while (names.hasNext())           
            _isEnabled = true;
        } // synchronized (TracerSingleton.class)
    } // public static enable()

    /**
     * Disabilita il tracciamento
     */
    public static void disable() {
        synchronized (TracerSingleton.class) {
            if (_loggerRegistry == null)
                return;
            //ottengo un vettore con tutte le key = NAME della HashMap 
            Iterator names = (_loggerRegistry.keySet()).iterator();
            while (names.hasNext()) {
                IFaceLogger logger =
                    (IFaceLogger) _loggerRegistry.get((String) names.next());
                try {
                    logger.disable();
                } // try
                catch (Exception ex) {
                    ex.printStackTrace();
                } // catch (Exception ex)
            } //while (names.hasNext())
            _isEnabled = false;
        } // synchronized (TracerSingleton.class)
    } // public static disable()

    /**
     * Verifica se il tracer &egrave; abilitato a tracciare.
     * @return <code>boolean</code>
     */
    public static boolean isEnabled() {
        synchronized (TracerSingleton.class) {
            return _isEnabled;
        } // synchronized (TracerSingleton.class)
    } // public static boolean isEnabled()

    /**
     * Questo metodo viene invocato per effettuare il tracciamento.
     * @param application la stringa che descrive il punto nel quale  avvenuta la
     * chiamata per il tracciamento.
     * @param severity codice della severity.
     * @param message descrizione del messaggio.
     */
    public static void log(String application, int severity, String message) {
        if (!_isEnabled)
            return;
        // prima provo se esiste un logger con questo nome.
        IFaceLogger logger = (IFaceLogger) _loggerRegistry.get(application.toUpperCase());  
        
        if (logger == null){
        	// utilizzo il default;
        	logger=_defaultLogger;
        }
        // Se non ho trovato nessun logger utilizzo la System.out.print
        if (logger == null)
            System.out.print(
                "TracerSingleton::log:" + application + ": " + message);
        else {
            Monitor monitor = null;
            try {
                monitor =
                    MonitorFactory.start(
                        "common.tracing."
                            + logger.getName()
                            + ".string");
                logger.trace(logger.getName(), severity, message);
            } // try
            catch (Exception ex) {
                ex.printStackTrace();
            } // catch (Exception ex)
            finally {
                if (monitor != null)
                    monitor.stop();
            } // finally
        } // if (logger == null)
    } // public static void log(String application, int severity, String message)

    /**
     * Questo metodo viene invocato per effettuare il tracciamento.
     * @param application la stringa che descrive il punto nel quale  avvenuta la
     * chiamata per il tracciamento.
     * @param severity codice della severity.
     * @param message descrizione del messaggio.
     * @param ex istanza di una eccezione.
     */
    public static void log(
        String application,
        int severity,
        String message,
        Exception ex) {
        if (!_isEnabled)
            return;
        IFaceLogger logger = (IFaceLogger) _loggerRegistry.get(application.toUpperCase());  
        
        if (logger == null){
        	// utilizzo il default;
        	logger=_defaultLogger;
        }
        if (logger == null)
            System.out.print(
                "TracerSingleton::log:" + application + ": " + message);
        else {
            Monitor monitor = null;
            try {
                if (logger.testToTrace(severity)) {
                    monitor =
                        MonitorFactory.start(
                            "common.tracing."
                                + logger.getName()
                                + ".exception");
                    StringWriter exStringWriter = new StringWriter();
                    PrintWriter exPrintWriter = new PrintWriter(exStringWriter);
                    ex.printStackTrace(exPrintWriter);
                    String errorMessage =
                        message
                            + ConfigSingleton.LINE_SEPARATOR
                            + exStringWriter.toString();
                    logger.trace(logger.getName(), severity, errorMessage);
                } // if (logger.testToTrace(severity))
            } // try
            catch (Exception e) {
                e.printStackTrace();
            } // catch (Exception e)
            finally {
                if (monitor != null)
                    monitor.stop();
            } // finally
        } // if (logger == null)
    } // public static void log(String application, int severity, String message, Exception ex)

    /**
     * Questo metodo viene invocato per effettuare il tracciamento.
     * @param application la stringa che descrive il punto nel quale  avvenuta la
     * chiamata per il tracciamento.
     * @param severity codice della severity.
     * @param message descrizione del messaggio.
     * @param xmlObject istanza di un oggetto rappresentabile in XML.
     */
    public static void log(
        String application,
        int severity,
        String message,
        XMLObject xmlObject) {
        if (!_isEnabled)
            return;
        IFaceLogger logger = (IFaceLogger) _loggerRegistry.get(application.toUpperCase());  
        
        if (logger == null){
        	// utilizzo il default;
        	logger=_defaultLogger;
        }
        if (logger == null)
            System.out.print(
                "TracerSingleton::log:" + application + ": " + message);
        else {
            Monitor monitor = null;
            try {
                if (logger.testToTrace(severity)) {
                    monitor =
                        MonitorFactory.start(
                            "common.tracing."
                                + logger.getName()
                                + ".xmlobject");
                    String newMessage =
                        message + ConfigSingleton.LINE_SEPARATOR;
                    if (xmlObject == null)
                        newMessage += "stream XML nullo";
                    else
                        newMessage += xmlObject.toXML(false);
                    logger.trace(logger.getName(), severity, newMessage);
                } // if (logger.testToTrace(severity))
            } // try
            catch (Exception e) {
                e.printStackTrace();
            } // catch (Exception e)
            finally {
                if (monitor != null)
                    monitor.stop();
            } // finally
        } // if (logger == null)
    } // public static void log(String application, int severity, String
} // public final class TracerSingleton
